;																
; 1WIRE-RS232-INTERFACE fr ATmega8								
; Bernhard.Erfurt@gmx.de										
; 03/2012														
;																
; PROTOKOLL:													
; "*" + KOMMANDO1 + KOMMANDO2									
;																
; KOMMANDO1+KOMMANDO2:											
; "V"+DUMMY - RS232 Verbindungstest 	Antwort=> "V"			
; "T"+DUMMY - 1WIRE BUS/PIN-STATUS		Antwort=> 0/1			
; "L"+DUMMY - 1WIRE BUS/PIN auf LOW		Antwort=> "L"			
; "S"+DUMMY - 1WIRE BUS/PIN auf HIGH	Antwort=> "S"			
; "P"+DUMMY - RESET-PRESENCE			Antwort=> 0/1(ERROR)	
;																
; "W"+BYTE 	- 1WIRE WRITE BYTE			Antwort=> "W"			
; "R"+DUMMY - 1WIRE READ  BYTE			Antwort=> BYTE			
; "I"+0/1 	- 1WIRE WRITE BIT			Antwort=> "I"			
; "D"+DUMMY - 1WIRE READ  BIT			Antwort=> 0/1			
; "Z"+DUMMY - 1WIRE READ 2xBIT(z.B.A+B)	Antwort=> 0/1/2/3		
;																


;												
; Anschlussbelegung PINS						
;												
; PB0 ==> 										
; PB1 ==> 										
; PB2 ==> 										
; PB3 ==> 										
; PB4 ==> 										
; PB5 ==> 										
; PB6 ==> Quarz									
; PB7 ==> Quarz									
;												
; PC0 ==> 										
; PC1 ==> 										
; PC2 ==> 										
; PC3 ==> 										
; PC4 ==> 										
; PC5 ==> 										
;												
; PD0 ==> C-RX									
; PD1 ==> C-TX									
; PD2 ==> 1WIRE									
; PD3 ==> 										
; PD4 ==> LED BLAU								
; PD5 ==> 										
; PD6 ==> 										
; PD7 ==> 										
;												


; ATmega8 										
.include "m8def.inc"
; KONSTANTEN									
.equ TAKT					=1000000			; 1 MHz	
; USART											
.equ BAUD					=4800
; 1-WIRE-DATENBUS								
.equ ONE_WIRE_DDR			= DDRD
.equ ONE_WIRE_PORT			= PORTD
.equ ONE_WIRE_PIN			= PIND
.equ ONE_WIRE_PORTBIT		= 2
; LEDS 											
.equ LED_D_BLAU			= 4
; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN					=0x0060			; 0x0060 START Adresse 		

.equ adr_RS232_ANZAHL_BYTES		=0x0060+100
.equ adr_RS232_KOMMANDO1		=0x0060+101
.equ adr_RS232_KOMMANDO2		=0x0060+102


; REGISTER										

.def NULL 	= R12
.def EINS 	= R14
.def VOLL	= R15

.def temp 	= R16
.def temp1 	= R17
.def temp2 	= R18
.def temp3 	= R19
.def temp4 	= R20
.def temp5 	= R21
.def temp6 	= R22
.def temp7 	= R23
.def temp8 	= R24
.def temp9 	= R25





.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp INTERRUPT_ERROR	;rjmp EXT_INT0   ; IRQ0 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF   	  	; Timer1 Overflow Handler       
rjmp INTERRUPT_ERROR	;rjmp TIM0_OVF   ; Timer0 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp BYTE_EINLESEN 		; USART RX Complete Handler     						
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	


; Unkorrekten Interrupt Aufruf 						
INTERRUPT_ERROR:
	cbi PORTD,LED_D_BLAU	; BLAU	
	rcall wait_100ms
	rcall wait_100ms
	sbi PORTD,LED_D_BLAU	; BLAU	
	rjmp INTERRUPT_ERROR

RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; SRAM lschen										
	rcall SRAM_CLEAR
; PORT B initialisieren								
	ldi temp, 0b00000000	;
	out DDRB, temp
; PORT B setzen										
	ldi temp, 0b00000000	;
	out PORTB, temp				
; PORT C initialisieren								
	ldi temp, 0b00000000	;
	out DDRC, temp
; PORT C setzen										
	ldi temp, 0b00000000	;
	out PORTC, temp				
; PORT D initialisieren								
	ldi temp, 0b00010010	; LED
	out DDRD, temp
; PORT D setzen										
	ldi temp, 0b00010010	;
	out PORTD, temp				
; STARTWERTE										
	; NULL				
	clr NULL
	; EINS				
	ldi temp,1
	mov EINS,temp
	; VOLL				
	ldi temp,255
	mov VOLL,temp

; USART_INITIALISIEREN		
	rcall USART_INITIALISIEREN
; ONE_WIRE_INITIALISIERUNG	
	rcall ONE_WIRE_INITIALISIERUNG
; interrupts freigeben								
	SEI
	cbi PORTD,LED_D_BLAU	; BLAU	




; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	LDS temp,(adr_RS232_ANZAHL_BYTES)
	cpi temp,3
	breq LOOP_w
	rjmp LOOP
LOOP_w:
	CLI						; INTERRUPTS sperren	
	cbi PORTD,LED_D_BLAU	; BLAU					
	rcall AUSWERTUNG		; AUSWERTUNG			
	sbi PORTD,LED_D_BLAU	; BLAU					
	STS (adr_RS232_ANZAHL_BYTES),NULL
	SEI						; INTERRUPTS freigeben	


rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
AUSWERTUNG:
	LDS temp1,(adr_RS232_KOMMANDO1)
	; VERBINDUNGSTEST		
	cpi temp1,'V'
	breq AUSWERTUNG_VERBINDUNGSTEST
	; PIN-TEST				
	cpi temp1,'T'
	breq AUSWERTUNG_PIN_TEST
	; PIN auf LOW ziehen	
	cpi temp1,'L'
	breq AUSWERTUNG_PIN_LOW
	; STRONG-PULLUP (VOLL POWER auf PIN)
	cpi temp1,'S'
	breq AUSWERTUNG_MAX_POWER
	; RESET-PRESENCE		
	cpi temp1,'P'
	breq AUSWERTUNG_RESET_PRESENCE
	; WRITE-BYTE			
	cpi temp1,'W'
	breq AUSWERTUNG_WRITE_BYTE
	; READ-BYTE				
	cpi temp1,'R'
	breq AUSWERTUNG_READ_BYTE
	; BIT-WRITE				
	cpi temp1,'I'
	breq AUSWERTUNG_BIT_WRITE
	; BIT-READ				
	cpi temp1,'D'
	breq AUSWERTUNG_BIT_READ
	; 2xBIT-READ			
	cpi temp1,'Z'
	breq AUSWERTUNG_2BIT_READ

	; ERROR					
	ldi temp1,255
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; VERBINDUNGSTEST																
AUSWERTUNG_VERBINDUNGSTEST:
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF EINGANG	
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; PULL-UP aus	
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; PIN-TEST																		
AUSWERTUNG_PIN_TEST:
	; SAMPLES-PIN abfragen			
	sbic (ONE_WIRE_PIN),(ONE_WIRE_PORTBIT)	; springe, wenn BIT(PIN) low	
	rjmp AUSWERTUNG_PIN_TEST1
AUSWERTUNG_PIN_TEST0:
	ldi temp1,0	; 0 zurcksenden	
	rjmp AUSWERTUNG_FERTIG
AUSWERTUNG_PIN_TEST1:
	ldi temp1,1	; 1 zurcksenden	
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; PIN gegen GND																	
AUSWERTUNG_PIN_LOW:
	sbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; AUF LOW		
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; VOLL POWER auf PIN															
AUSWERTUNG_MAX_POWER:
	sbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; AUF HIGH		
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; RESET-PRESENCE																
AUSWERTUNG_RESET_PRESENCE:
	; Port auf EINGANG + HIGH		
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF EINGANG	
	rcall wait_1ms
	; PORT auf AUSGANG und LOW		
	sbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; AUF LOW		
	rcall WAIT_500us
	; Port auf EINGANG + HIGH		
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF EINGANG	
	; wenn LOW, dann ERROR			
	NOP
	NOP
	NOP
	sbis (ONE_WIRE_PIN),(ONE_WIRE_PORTBIT) 	; "skip if..."
	rjmp AUSWERTUNG_RESET_PRESENCE_ERROR
	; Pause 50s 					
	rcall WAIT_50us
	; SAMPLES						
	sbic (ONE_WIRE_PIN),(ONE_WIRE_PORTBIT) 	; "skip if..."
	rjmp AUSWERTUNG_RESET_PRESENCE1
AUSWERTUNG_RESET_PRESENCE0:
	ldi temp1,0		; 0 zurcksenden	
	rjmp AUSWERTUNG_FERTIG
AUSWERTUNG_RESET_PRESENCE1:
	ldi temp1,1		; 1 zurcksenden	
	rjmp AUSWERTUNG_FERTIG
AUSWERTUNG_RESET_PRESENCE_ERROR:
	ldi temp1,255	; 255 zurcksenden	
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; WRITE BYTE																	
AUSWERTUNG_WRITE_BYTE:
	LDS temp1,(adr_RS232_KOMMANDO2)	; BYTE					
	rcall ONE_WIRE_DATA_WRITE		; 1-WIRE senden			
	ldi temp1,'W'					; 'W' zurcksenden		
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; READ BYTE																		
AUSWERTUNG_READ_BYTE:
	rcall ONE_WIRE_DATA_READ		; OUT: temp1			
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; BIT WRITE																		
AUSWERTUNG_BIT_WRITE:
	LDS temp1,(adr_RS232_KOMMANDO2)	; BYTE					
	LSR temp1						; 1xrechts C rollt raus	
	rcall ONE_WIRE_DATA_WRITE_BIT	; INP: Carry			
	ldi temp1,'I'					; 'I' zurcksenden		
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; BIT_READ																		
AUSWERTUNG_BIT_READ:
	rcall ONE_WIRE_DATA_READ_BIT		; OUT: CARRY
	clr temp1
	ROL temp1 							; logisch links ber Carry
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; 2xBIT_READ																	
AUSWERTUNG_2BIT_READ:
	clr temp1
	rcall ONE_WIRE_DATA_READ_BIT		; OUT: CARRY				
	ROR temp1  							; logisch rechts ber Carry	
	rcall ONE_WIRE_DATA_READ_BIT		; OUT: CARRY				
	ROR temp1  							; logisch rechts ber Carry	
	LSR temp1							; 6x logisch rechts			
	LSR temp1
	LSR temp1
	LSR temp1
	LSR temp1
	LSR temp1
	rjmp AUSWERTUNG_FERTIG
; ------------------------------------------------------------------------------
; 1 BYTEper RS232 zurcksenden		
AUSWERTUNG_FERTIG:
	rcall BYTE_SENDEN_TEMP1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "INTERFACE_USART.asm"				; USART							
.include "INTERFACE_SONSTIGES.asm"			; sonstiges						
.include "INTERFACE_WAIT_01MHZ.asm"		;  1 MHz						
.include "INTERFACE_1WIRE.asm"
